/*****************************************************************************/
/*									     */
/*									     */
/*	Xpack version 1.00 -- module xpack.h				     */
/*									     */
/*	This file is included by all sources of xpack.			     */
/*	Written by Michael Bischoff (mbi@mo.math.nat.tu-bs.de)		     */
/*	November-1994							     */
/*	see COPYRIGHT.xpack for Copyright details			     */
/*									     */
/*									     */
/*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <limits.h>

/* #include "xpack.h" */

#define ALLOW_90	1
#define ALLOW_180	2
#define ALLOW_MIRROR	4

struct tile {
	int count;
	int type;
	int width;
	int height;
	int score;
	char **map; /* or NULL */
	char name[8];
};
struct position {
	int x;	/* x = -1 means not on board */
	int y;
	int type;	/* rotated / mirrored? */
	int tilenr;
};
struct problem {
	int maxtile;
	struct tile *t;
	int totaltiles;	/* of 1...#maxtile */
	struct position *position;
	int **board;
	int maxscore;
	char description[32];
};

extern struct problem P;
extern int state;
extern struct position *active;
extern struct active {
    int tilenr, type, w, h;
    char **map;
} a;

void free_problem(void);
void generate_rectangle_problem(int n, int f, int d, int width, int height);
void clear_position(void);
void make_simple_solution(void);

int place_active(int x, int y);
void discard_active(void);
void activate_free(int k);
void activate_used(int k);

void popup_active(int, int, int, int);
int place_and_redraw(int, int);
void activate_and_redraw(int);

/* defaults for user configurable directories */
#ifndef XPACKDIR		/* directory where to get the game data files */
#define XPACKDIR		"/usr/games/lib/xpack"
#endif
#ifndef XPACKSAVE	/* directory where to save moves of solved levels */
#define XPACKSAVE	"/var/games/xpack"
#endif

#define MAXSAVEFILELEN	200	/* pathname length including trailing zero */
#define MAXXPACKDIRLEN	 99	/* pathname length of the xpack directory */
#define MAXFILENAMELEN	 30	/* filename length of keyboard and message file */

/* maximum board dimensions, inclusive outer borders (tunable) */
#define MAXCOL          32	/* max x dim. */
#define MAXROW          22	/* max y dim. */
/* other maximum settings (tunable) */
#define MAXWALLS	32	/* max. different floor types */
#define MAXOBJECTS	32	/* max. different object types */
#define MAXINSTANCES	256	/* max. number of objects */

/* global string variables */
extern char username[256];   /* which user to blame for success					*/
extern const char *savedir;  /* in which directory should saved games be stored (XPACKSAVE)	*/
extern const char *xpackdir;  /* from which directory to read the level files (default: XPACKDIR) */
extern const char *langdir;		/* a string (empty?) defining the language */
extern const char *rulepool[16];
extern int highscore[300];
extern int pushcost;
extern int movecost;
extern void (*lastcmd)(void);
extern int numeric_arg;
extern int DX, DY;

#ifndef EXIT_FAILURE	/* poor old SUN's */
#define EXIT_FAILURE (-1)
#endif

#ifndef max
#define max(a, b)     ((a) > (b) ? (a) : (b))
#define min(a, b)     ((a) < (b) ? (a) : (b))
#endif

typedef int boolean;            /* just one bit of information           */
struct key_action {
    char *string;
    void (*action)(void);
    struct key_action *next;
};

#define True		1
#define False		0

#if 0
/* obsolete */
#define XPACK1MAGIC1	0xb5
#define XPACK1MAGIC2	0xa0
#define ARR_UP		0x100
#define ARR_LEFT	0x101
#define ARR_DOWN	0x102
#define ARR_RIGHT	0x103

#endif

#define Disable 0
#define Enable	1
#define EnableAndRedraw 2


struct walls {
    int chr;	/* character for representation		*/
    int pic;	/* picture number			*/
    int enter;
    int leave;
    int mask;
    int effect;
};

/* definitions for the effect field */
/* all effects plus 100: square will turn to normal floor if touched. */
#define E_ONCE		100	/* add this to make one-time effects */
#define E_NOTHING	0
#define E_TURN_CCW	1
#define E_TURN_180	2
#define E_TURN_CW	3
#define E_DEST		4	/* no effect, but required by finished() */
#define E_EXIT		5	/* dito */
#define E_ADDPOWER	6
#define E_SUBPOWER	7
#define E_TELEPORT	8

/* addstrength, teleporters */

struct objects {
    int chr;	/* character for representation		*/
    int pic;	/* picture number			*/
    int movedir;
    int pushdir;
    int weight;
    int power;
    int mask;
    int score;
};

struct game {
    int numrows;	/* game size */
    int numcols;
    int x;		/* player pos */
    int y;
    int n_pushes;	/* counter */
    int n_moves;
    int stored_moves;
    int bookmark;
    int finished;
    int level;
    int score;
    const char *type;
    int macroStart, macroEnd;
    int macro_x, macro_y;
};


extern char levelcomment[100];
extern char levelauthor[100];
extern int gamegraphic;
extern int maxlevel;		/* maximum level number for this type of game */
extern int nwalls, nobjects, ninstances;
extern struct objects objects[MAXOBJECTS];
extern struct walls walls[MAXWALLS];	/* wall types */
extern struct game game;
extern struct walls *map[MAXROW][MAXCOL];
extern struct objects *obj[MAXROW][MAXCOL];
extern struct objects instance[MAXINSTANCES];
extern char *movetab;
extern int numalloc;

void set_type(int);

/* function prototypes */
/* parse.c */
void ParseDefinitionFile(void);	/* read definitions */
void ParseMapFile(void);	/* read a level */
void OrgLevel(void);		/* restart game */

/* tools.c */
int isqrt(long);
int ilog2(long);
void fatal(const char *, ...);
void *malloc_(size_t);
void *calloc_(size_t, size_t);
void *realloc_(void *, size_t);
void free_(void *);
char *strsav(const char *);

/* main.c */
void change_rules(const char *);
int compute_score(void);
int finished(void);

/* move.c */
void graphics_control(int);

void cmd_SetImp0(void);
void cmd_SetImp1(void);
void cmd_TryAll1(void);
void cmd_TryAll2(void);
void cmd_TryAll3(void);
void cmd_TryAll4(void);
void cmd_Describe(void);
void cmd_write_LaTeX(void);
void cmd_write_PostScript(void);

/* username.c */
void buildusername(const char *);

/* messages.c */
void read_message_file(const char *);
void add_keybinding(struct key_action **, const char *, const char *);
void read_keyboard_file(const char *);
void key_pressed(char *);

/* loadsave.c */
void cmd_ReadHighscores(void);
void WriteHighscores(void);
void switch_uid(int);
void setlangdir(void);
void load_game(const char *);
void save_game(const char *);
void link_game(const char *, const char *);

/* commands.c */
void cmd_LevelInfo(void);
void cmd_PlaceLargest(void);
void cmd_Heuristik1(void);
void rq_LeavePack(void);
void rq_RestartGame(void);
void cmd_RestartGame(void);
void cmd_SaveGame(void);
void cmd_ShowVersion(void);
void cmd_ShowScore(void);
void cmd_ShowArea(void);
void cmd_LoadGame(void);

/* Xaw-main.c */
void show_message(const char *str, ...);
void SetTitle(void);
void cmd_LeavePack(void);
void cmd_Confirm(void);
void cmd_Cancel(void);
void request_confirm(void (*)(void), const char *);
int main(int argc, char *argv[]);
/* void Force_Resize(XSize_t, XSize_t); */

/* X-widget.c */
/* void AskWidgetForResize(XSize_t, XSize_t); */

/* X-events.c */
void refresh_screen(void);
/* void button_press(XButtonPressedEvent *);
void key_press(XKeyPressedEvent *); */
void cmd_Resize(void);
/* void resize_event(XSize_t, XSize_t); */

/* X-gfx.c */
void sync_and_wait(void);
void NewLevel(int);
void init_layout(void);
/* void dotPaint(int, int, int, int); */
void doPaint(int, int, int, int);
/* void redraw_table(XExposeEvent *); */

/* dummy.c */
void cmd_debug(void);

/* xfopen.c */
FILE *zreadopen(const char *filename);
void zreadclose(FILE *fp);

extern int mouse_x, mouse_y, mouse_x0, mouse_y0;
